"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createIpcMainApiService = void 0;
const tslib_1 = require("tslib");
const Lib = tslib_1.__importStar(require("pubsub-to-rpc-api"));
const util_1 = require("./private/util");
const PM = tslib_1.__importStar(require("./private/model"));
const electron_require_1 = require("./private/electron-require");
const ipcMainEventEmittersCache = new WeakMap();
const ipcRendererEventEmittersCache = new WeakMap();
function createIpcMainApiService(createServiceArg) {
    const baseService = Lib.createService(createServiceArg);
    return {
        register(actions, options) {
            const { ipcMain = (0, electron_require_1.requireIpcMain)(), logger, } = options || {};
            const cachedEm = (ipcMainEventEmittersCache.get(ipcMain)
                ||
                    (() => {
                        const em = {
                            on: ipcMain.on.bind(ipcMain),
                            removeListener: ipcMain.removeListener.bind(ipcMain),
                            emit: ipcMain.emit.bind(ipcMain),
                        };
                        ipcMainEventEmittersCache.set(ipcMain, em);
                        return em;
                    })());
            return baseService.register(actions, cachedEm, {
                onEventResolver: (...[event, payload]) => {
                    return {
                        payload,
                        emitter: {
                            emit: (...args) => {
                                if (!event.sender.isDestroyed()) {
                                    event.reply(...args);
                                    return;
                                }
                                if (logger) {
                                    logger.debug(`[${PM.MODULE_NAME}]`, `Object has been destroyed: "sender"`);
                                }
                            },
                        },
                    };
                },
                logger,
            });
        },
        client(_a = {}) {
            var { ipcRenderer = (0, electron_require_1.requireIpcRenderer)() } = _a, _b = _a.options, _c = _b === void 0 ? {} : _b, { timeoutMs = PM.BASE_TIMEOUT_MS, logger: _logger_ = createServiceArg.logger || PM.LOG_STUB } = _c, // tslint:disable-line:variable-name
            callOptions = tslib_1.__rest(_c, ["timeoutMs", "logger"]);
            const logger = (0, util_1.curryOwnFunctionMembers)(_logger_, `[${PM.MODULE_NAME}]`, "createIpcMainApiService() [client]");
            const cachedEm = (ipcRendererEventEmittersCache.get(ipcRenderer)
                ||
                    (() => {
                        const em = {
                            on: ipcRenderer.on.bind(ipcRenderer),
                            removeListener: ipcRenderer.removeListener.bind(ipcRenderer),
                            emit: ipcRenderer.send.bind(ipcRenderer),
                        };
                        ipcRendererEventEmittersCache.set(ipcRenderer, em);
                        return em;
                    })());
            return baseService.caller({
                emitter: cachedEm,
                listener: cachedEm,
            }, Object.assign(Object.assign({}, callOptions), { timeoutMs,
                logger, onEventResolver: (...[/* event */ , payload]) => {
                    return { payload };
                } }));
        },
    };
}
exports.createIpcMainApiService = createIpcMainApiService;
//# sourceMappingURL=ipc-main-api-service.js.map