"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createWebViewApiService = void 0;
const tslib_1 = require("tslib");
const Lib = tslib_1.__importStar(require("pubsub-to-rpc-api"));
const pure_uuid_1 = tslib_1.__importDefault(require("pure-uuid"));
const util_1 = require("./private/util");
const PM = tslib_1.__importStar(require("./private/model"));
const electron_require_1 = require("./private/electron-require");
const ipcRendererEventEmittersCache = new WeakMap();
const webViewTagEventEmittersCache = new WeakMap();
const clientIpcMessageEventName = "ipc-message";
const clientIpcMessageListenerBundleProp = Symbol(`[${PM.MODULE_NAME}] clientIpcMessageListenerBundleProp symbol`);
function createWebViewApiService(createServiceArg) {
    const baseService = Lib.createService(createServiceArg);
    const clientIpcMessageOnEventResolver = (ipcMessageEvent) => {
        const [payload] = ipcMessageEvent.args;
        return { payload };
    };
    return {
        register(actions, { logger, ipcRenderer = (0, electron_require_1.requireIpcRenderer)(), } = {}) {
            const cachedEm = (ipcRendererEventEmittersCache.get(ipcRenderer)
                ||
                    (() => {
                        const em = {
                            on: ipcRenderer.on.bind(ipcRenderer),
                            removeListener: ipcRenderer.removeListener.bind(ipcRenderer),
                            emit: ipcRenderer.sendToHost.bind(ipcRenderer),
                        };
                        ipcRendererEventEmittersCache.set(ipcRenderer, em);
                        return em;
                    })());
            return baseService.register(actions, cachedEm, {
                logger,
                onEventResolver: (...[/* event */ , payload]) => {
                    return {
                        payload,
                        emitter: cachedEm,
                    };
                },
            });
        },
        client(webView, _a = {}) {
            var _b = _a.options, _c = _b === void 0 ? {} : _b, { timeoutMs = PM.BASE_TIMEOUT_MS, logger: _logger_ = createServiceArg.logger || PM.LOG_STUB } = _c, // tslint:disable-line:variable-name
            callOptions = tslib_1.__rest(_c, ["timeoutMs", "logger"]);
            const logger = (0, util_1.curryOwnFunctionMembers)(_logger_, `[${PM.MODULE_NAME}]`, "createWebViewApiService() [client]");
            const cachedEm = (webViewTagEventEmittersCache.get(webView)
                ||
                    (() => {
                        const em = {
                            on: (originalEventName, originalListener) => {
                                const ipcMessageListenerBundle = {
                                    uid: new pure_uuid_1.default(4).format(),
                                    created: new Date(),
                                    originalEventName,
                                    actualListener: [
                                        clientIpcMessageEventName,
                                        (ipcMessageEvent) => {
                                            if (ipcMessageEvent.channel !== originalEventName) {
                                                return;
                                            }
                                            originalListener(clientIpcMessageOnEventResolver(ipcMessageEvent).payload);
                                        },
                                    ],
                                };
                                webView.addEventListener(...ipcMessageListenerBundle.actualListener);
                                // TODO consider keeping actual listeners in a WeakMap<typeof originalListener, IpcMessageListenerBundleProp>
                                // link actual listener to the original listener, so we then could remove the actual listener
                                // we know that "listener" function is not locked for writing props as it's constructed by "pubsub-to-rpc-api"
                                originalListener[clientIpcMessageListenerBundleProp]
                                    = ipcMessageListenerBundle;
                                logger.debug(`[cache] add event listener`, JSON.stringify({
                                    originalEventName,
                                    uid: ipcMessageListenerBundle.uid,
                                    created: ipcMessageListenerBundle.created,
                                }));
                                return em;
                            },
                            removeListener: (...[originalEventName, originalListener]) => {
                                const ipcMessageListenerBundlePropAware = originalListener;
                                const ipcMessageListenerBundle = ipcMessageListenerBundlePropAware[clientIpcMessageListenerBundleProp];
                                if (!ipcMessageListenerBundle
                                    ||
                                        ipcMessageListenerBundle.originalEventName !== originalEventName) {
                                    return em;
                                }
                                const logData = JSON.stringify({
                                    originalEventName,
                                    uid: ipcMessageListenerBundle.uid,
                                    created: ipcMessageListenerBundle.created,
                                });
                                if (webView.isConnected) {
                                    webView.removeEventListener(...ipcMessageListenerBundle.actualListener);
                                    logger.debug(`[cache] remove event listener`, logData);
                                }
                                else {
                                    logger.debug(`[cache] remove event listener: skipped since "webView" is not attached to the DOM`, logData);
                                }
                                delete ipcMessageListenerBundlePropAware[clientIpcMessageListenerBundleProp];
                                return em;
                            },
                            emit: (...args) => {
                                if (webView.isConnected) {
                                    // tslint:disable-next-line:no-floating-promises
                                    webView.send(...args);
                                }
                                else {
                                    logger.debug(`"webView.send()" call skipped since "webView" is not attached to the DOM`);
                                }
                            },
                        };
                        webViewTagEventEmittersCache.set(webView, em);
                        return em;
                    })());
            return baseService.caller({ emitter: cachedEm, listener: cachedEm }, Object.assign(Object.assign({}, callOptions), { timeoutMs }));
        },
    };
}
exports.createWebViewApiService = createWebViewApiService;
//# sourceMappingURL=webview-api-service.js.map